#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html

# ------------
#| Change Log |
# ------------

# Plug-in to open GIMP's vectors or layer in 'Inkscape'. For Linux and Windows.

# Version 0.4.0 (2016/03/17) at http://gimpchat.com/viewtopic.php?f=9&t=13723
#    With translation possibility

# Version 0.4.2 (2016/06/16)
#  -Offer to save the arguments of the plug-in if a change in the folders args (SVG & Inkscape).
#   By making a folder in 'gimp.directory' called 'py_configs' & a list file.
#  -One running instance only.
#  -Produce a 'bail_out(msg)' function to streamline the code.
#  -Better management of the progress bar.
#  -Use the name given at the beginning to the SVG file for the render layer.
#  -Protect the plug-in exit and advise the user if no SVG file found after Inkscape,
#   or avoid rendering on a group layer (a crash).
		
# Version 0.5 MrQ (2023/07/19) remove create py_configs/Gimp_to_Inkscape.txt, 
#                              add option: Exported active layer & return as new image,
#                              change menu in Gimp to Filters/Render/Connection to Inkscape...(old menu: Vectors/Connection to Inkscape &...),
#                              changing the default "Exchange directory:" from "~/Desktop" to "tmp" in gimp.directory,
#                              remove the extension from the layer name (if exist).

from gimpfu import *
import sys, os
import gtk, gettext
import subprocess as sub

### global #####################################################################

### change it to your liking
GTI_menu_path = "<Image>/Filters/Render/"
### plug-in filename (thanks 'Ofnuts')
whoiam = os.path.abspath(sys.argv[0])
whoiamName = whoiam.split(os.sep)[-1][:-3]

# if not launch within GIMP
def mssgBox(mess):
    flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, mess)
    msgBox.run()
    msgBox.destroy()
try:
    from gimpshelf import shelf
except ImportError:
    print("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    # launch from GUI? use mssgBox(text)
    mssgBox("Note: GIMP registering is needed, '%s' is a plug-in for it.\n"%whoiam)
    sys.exit(1)

# for locale characters in windows file system
def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)

# initialize internationalization in a user-locale
locale_user_dir = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( whoiamName, locale_user_dir, unicode=True )


GTI_args = ["", "", 0, 3, True]

if os.name == 'nt':
    GTI_args[0] = "C:\\Program Files\\Inkscape\\bin\\inkscape.exe"

else:
    GTI_args[0] = "/usr/bin/inkscape"

#GTI_args[1] = os.path.expanduser("~/Desktop")
GTI_args[1] = gimp.directory + os.sep + "tmp"

### main function ##############################################################

def open_path_or_layer_in_inkscape(image, layer, filename, ink, outputFolder, sel, schel, rem):


    ### utility functions
    def bail_out(msg):
        """
        Display a message in the 'Error console' and bail out
        """
        gimp.message(msg)
        image.undo_group_end()
        gimp.progress_init(_("Plug-in had input error!"))
        pdb.gimp_progress_update(0.2)
        shelf[whoiamName] = False

    def render_svg_layer(img, svg_f, sep):
        """
        Rendering a 'Scalable Vector Graphics' file on the image top layer
        """
        resol_x = img.resolution[0]
        # image = pdb.file_svg_load(filename, raw_filename, resolution, width, height, paths)
        fileImage = pdb.file_svg_load(svg_f, svg_f, resol_x, 0, 0, 2)
        pdb.gimp_edit_copy(fileImage.layers[0])
        pdb.gimp_image_delete(fileImage)
        # pasted probl (RuntimeError) with group layer like in ArrowsCreator
        if hasattr(img.layers[0],"layers"): # Type = Group layer
            layer = img.new_layer('GTI', img.width, img.height, 2*img.base_type+1, 100, 0)
            pdb.gimp_layer_set_offsets(layer, 0, 0)
            pasted = pdb.gimp_edit_paste(layer, True)
            group_bool = True
        else:
            pasted = pdb.gimp_edit_paste(img.layers[0], True)
            group_bool = False
        pdb.gimp_floating_sel_to_layer(pasted)
        # set name of this layer base on filename
        newname = _('Render of ') + svg_f.split(sep)[-1]
        img.layers[0].name = newname
        if group_bool: pdb.gimp_image_remove_layer(img, layer)
        return True

    ### start main function,
    image.undo_group_start()
    shelf[whoiamName] = True
	
    active_layer = pdb.gimp_image_get_active_layer(image)	
    sourcename = active_layer.name
    if '.' in sourcename:
        split = sourcename.split(".", 1)
        sourcename = split[0]
        layer.name = sourcename	
		
    # in GIMP - armouring (if no vectors 'layer' it won't start)
    if os.access(sys_file(outputFolder), os.W_OK):
        # construct the filename of vector exchange file in 'utf-8'
        if sel == 2:
           xchange_utf = outputFolder + os.sep + layer.name + ".png"
           gimp.progress_init(_("Saving to '") + xchange_utf)
        else: 
           xchange_utf = outputFolder + os.sep + layer.name + ".svg"
           gimp.progress_init(_("Saving to '") + xchange_utf)		 
    else:
        bail_out(_("ERROR: folder '%s' isn't writable, plug-in has terminated.")\
            %outputFolder)
        return
    if os.name == 'nt': tail = "inkscape.exe"
    else: tail = "inkscape"
    if not (ink.endswith(tail) and os.access(ink, os.X_OK)) :
        bail_out(_("ERROR: executable '%s' pose problem, plug-in has terminated.")\
            %ink)
        return

    # export which path;
    if sel == 0:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, None)
    if sel == 1:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, image.active_vectors)

##################################################################################################################	
		
		
    if sel == 2:
        xchange_utf = outputFolder + os.sep + layer.name + ".png"
        pdb.file_png_save(image, layer, xchange_utf, xchange_utf, 1, 9, 1, 1, 0, 1, 1, run_mode=RUN_NONINTERACTIVE)
		
###################################################################################################################	
    GTI_args[2] = sel
    #GTI_args[1] = saveas

    # plus inkscape with filename, 'ink' should be ASCII. Same file after inkscape?
    gimp.progress_init(_("Connecting to Inkscape..."))
    pdb.gimp_progress_update(0.5)
    xchange_sys = sys_file(xchange_utf) # for Python, Win file system not 'utf-8'
    file_befor = os.stat(xchange_sys)
    p = sub.Popen([ink, xchange_sys], stdin=sub.PIPE, stdout=sub.PIPE, stderr=sub.STDOUT)
    child_output, child_error = p.communicate()
    # In inkscape saving a XCF file is also possible?

    # after inkscape, GIMP import vectors and/or render.
    if os.access(xchange_sys, os.F_OK):
        file_after = os.stat(xchange_sys)
		
        #if schel == 1 and file_befor != file_after:     # vectors
        if schel == 1 :     # vectors		
            if sel == 2:
                xchange_utf = outputFolder + os.sep + layer.name + ".svg"			
            gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
	
        if schel == 2 :   # both
            if sel == 2:
                xchange_utf = outputFolder + os.sep + layer.name + ".svg"
            #if file_befor != file_after:
            gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
            render_svg_layer(image, xchange_utf, os.sep)

			
        if schel == 3 :   # render new image
            xchange_utf = outputFolder + os.sep + layer.name + ".svg"
            #active_layer = pdb.gimp_image_get_active_layer(image)
            w = pdb.gimp_image_width(image)
            h = pdb.gimp_image_height(image)

            new_img = pdb.gimp_image_new(w, h, 0)
            pdb.gimp_display_new(new_img)
		
            gimp.vectors_import_from_file(new_img, xchange_utf, TRUE, TRUE)	
            nlay=pdb.gimp_file_load_layer(new_img, xchange_utf)
            pdb.gimp_image_insert_layer(new_img, nlay, None, 0)			
        # cleanup vector file?
        if rem :
            os.remove(xchange_sys)
    else: gimp.message(_("WARNING: no SVG file found! So nothing after Inkscape."))
    GTI_args[3] = schel
    GTI_args[4] = rem

    gimp.displays_flush()
    shelf[whoiamName] = False
    image.undo_group_end()
    gimp.progress_init(_("Plug-in end normally."))
    pdb.gimp_progress_update(1.0)

    ## manage the arguments file for changed folders

### GIMP registering & UI ######################################################

register(
    "open-path-or-layer-in-inkscape",
    _("This plug-in opens: active or all GIMP vectors or active layer\n")\
    +_("in Inkscape, then can acquire and/or render them.\n")\
    +_("--------------- Ver.0.5 MrQ Instructions: ----------------------------\n")\
    +_("Before in GIMP, select path or layer (name may use special characters).")\
    +_("After start, this name + '.svg' (path) or + '.png' (layer) will be the save file for exchange directory.\n")\
    +_("In Inkscape: in windows 'png bitmap image import' select Image DPI: 'Default import resolution', save changes as\n")\
    +_("name+'.svg' and close. This return to GIMP and end the plugin.\n")\
    +"------------------------------------------------------------------------------\n"\
    +_("Plug-in file-name:\n")+whoiam,
    "Open active or all vector layers to Inkscape and return to GIMP with a vectors\n"\
        +"re-acquisition and/or rendering possibility- V 0.5 MrQ",
    "MareroQ and rob_brz1",
    "GPLv3",
    "2019/23",
    _("Connection to Inkscape..."),
    "RGB*,GRAY*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "layer", "Input layer", None),		
        (PF_VECTORS, "filename", "Vector exchange file:", None),
        (PF_FILENAME, "ink", _("Inkscape executable file: "), GTI_args[0]),
        (PF_DIRNAME, "outputFolder", _("Exchange directory:"), GTI_args[1]),
        (PF_OPTION, "sel", _("Exported:"), GTI_args[2], (_("All paths"),_("Active path"),_("Active layer"))),
        #(PF_OPTION, "saveas", _("Save layer as:"), GTI_args[2], (_("PNG"),_("BMP"))),		
        (PF_OPTION, "schel", _("Return to GIMP with:"), GTI_args[3], (_("Nothing"),_("Vectors"),_("SVG Render"),_("SVG Rendering as New Image"))),
        (PF_BOOL, "rem", _("Delete SVG file? "), GTI_args[4])
    ],
    [],
    open_path_or_layer_in_inkscape,
    menu = GTI_menu_path,
    domain = (whoiamName, locale_user_dir)
    )

main()
